#include "inet.h"
#include "segment.h"

/* external functions */
extern int      CommInit();
extern void     CommQuit();
extern int      GetDataSize(int, double, double, double, double, int* ,
		        int* , int* , int*, void** );
extern int      RemoteData(int, int, int, int, int, int, void* , void** );
extern int      errno;
extern char     *sys_errlist[];

/* globals for socket numbers */
static int datasock, ctrlsock; /* data and control socket descriptors */

main(int argc, char **argv)

{
   int             size = 0;
   int             ret, data_type, start_row, start_column, rows, columns,
                   scale;
   double          south, west, north, east;
   int             nelements, datasize, bufsize;
   void            *data = NULL;
   void            *refptr;
   FILE            *fdmap, *fdcoo;

   refptr = (void *)malloc(sizeof(Misc));

/* initialize communication channels */
   if (CommInit() == CONNECT_ERROR)
      exit(ERROR);

/* open the map output file */
   if ((fdmap = fopen("map", "w")) == NULL)
   {
      /* open map segment output file */
      printf("ERROR: Could not open map output file %s\n", "map");
      CommQuit();
      exit(ERROR);
   }

/* open the input file for coordinates */
   if ((fdcoo = fopen("coord.dat", "r")) == NULL)
   {
      printf("ERROR: Could not open input file %s\n", "coord.dat");
      CommQuit();
      exit(ERROR);
   }

   while (TRUE) {
     /* read coordinates from input file */
     fscanf(fdcoo, "%d %lf %lf %lf %lf\n", &data_type, &south, &west, 
	    &north, &east);
     
     /* convert coordinate format and calculate data size */
     ret = GetDataSize(data_type, south, west, north, east, &start_row,
		       &start_column, &rows, &columns, &refptr);
     if ((ret == DATA_ERROR) || (ret == RANGE_ERROR) || (ret == ERROR))
       break;
     if (south == -1.0) break; /* done - NOTE: the call GetDataSize must
				be made before quitting. This causes the
				server to clean up on its side */
     /* debug */
      fclose(fdcoo);
	  fclose(fdmap);
	  CommQuit();
	  exit(OK); 
     
     /* now get the data from server */
     switch(data_type) {
     case DEM:
       bufsize = DEM_bytes;
       break;
     case LULC:
       bufsize = Image_bytes;
       break;
     case LANDSAT:
       bufsize = Image_bytes;
       break;
     case DEM_LULC:
       bufsize = DEM_bytes + Image_bytes;
       break;
     case DEM_LANDSAT:
       bufsize = DEM_bytes + Image_bytes;
       break;
     case IMAGE3D:
       bufsize = 0;
       break;
     default:
       bufsize = 0;
       break;
     }
     nelements = rows * columns;
     datasize = nelements * bufsize;
     data = (void *)malloc(datasize);
     memset(data, '\0', datasize);
     ((Misc *)refptr)->datasize = datasize;
     ret = RemoteData(data_type, start_row, start_column, rows, columns,
		      scale, data, &refptr);
     if(data) free (data);
     if ((ret == DATA_ERROR) || (ret == RANGE_ERROR) || (ret == ERROR))
       break;

     rewind(fdmap);
     printf("Writing map segment to file: %s ... \n", "map");
     fwrite(data, datasize, 1, fdmap);
     /*
	if (data_type == 6)
	fwrite(data, datasize, 1, fdmap);
	else
	fwrite(data, datasize, 1, fdmap);
	*/
   }

   /* close all io streams */
   fclose(fdcoo);
   fclose(fdmap);
     
   /* terminate all communications */
   CommQuit();
   
   exit(OK);

 }
